#include "kana_editor.h"
// #include "helperwidget.h"
#include <QListWidget>
#include <QtDebug>
#include <QtGui>
#include <QtCore>
#include "KanaXmlHandler.h"


KanaEditor::KanaEditor(QWidget *parent) : QWidget(parent){
	setupUi(this);
// 	helper = new HelperWidget(editor);
// 	helper->hide();
	cursor_start_position = 0;
	cursor_end_position = 0 ;
	currentRomanjiWord = "";
	currentKanaWord = "";
	dataFileName = ":/data.xml";
	
	helpListWidget = new QListWidget(editor);
	connect(helpListWidget, SIGNAL(itemActivated(QListWidgetItem *)), this , SLOT(helpListWidget_itemActivated(QListWidgetItem *)) );
	
	QRect cursor_position = editor->cursorRect();
	QPoint br = cursor_position.bottomRight();
	helpListWidget->setGeometry( br.x() , br.y() , editor->width()/3 , editor->height()/3 );
	
	sentenceRootItem = new KanaListWidgetItem(helpListWidget,"",tr("Full words"));
	prepareRootItem(sentenceRootItem);
	hiraganaRootItem = new KanaListWidgetItem(helpListWidget,"",tr("Hiragana"));
	prepareRootItem(hiraganaRootItem);
	katakanaRootItem = new KanaListWidgetItem(helpListWidget,"",tr("Katakana"));
	prepareRootItem(katakanaRootItem);
	kanjiRootItem = new KanaListWidgetItem(helpListWidget,"",tr("Kanji"));
	prepareRootItem(kanjiRootItem);
	
	kanaList = new QList<KanaListWidgetItem *>;
	kanaWordList = new QList<KanaListWidgetItem *>;
	
// 	loadData(dataFileName);
	
	
	helpListWidget->hide();
	editor->setFocus(Qt::ActiveWindowFocusReason);
	
	QObject::connect(editor, SIGNAL(copyAvailable(bool)), this, SIGNAL(copyAvailable(bool)) );
	QObject::connect(editor, SIGNAL(redoAvailable(bool)), this, SIGNAL(redoAvailable(bool)) );
	QObject::connect(editor, SIGNAL(undoAvailable(bool)), this, SIGNAL(undoAvailable(bool)) );
}

void KanaEditor::setDataFile(const QString &fileName){
	dataFileName = fileName;
	QFile file(fileName);
	QXmlInputSource inputSource(&file);
	QXmlSimpleReader reader;
	kanaList->clear();
	KanaXmlHandler handler(kanaList);
	reader.setContentHandler(&handler);
	reader.setErrorHandler(&handler);
	reader.parse(inputSource);
	qDebug() << "kanaList have the following size after XML being parsed : " << kanaList->size();
}

QString KanaEditor::getDataFile(){
	return dataFileName;
}

void KanaEditor::editorCopy(){
	editor->copy();
}

void KanaEditor::editorCut(){
	editor->cut();
}

void KanaEditor::editorPaste(){
	editor->paste();
}

void KanaEditor::editorUndo(){
	editor->undo();
}

void KanaEditor::editorRedo(){
	editor->redo();
}

void KanaEditor::editorClear(){
	editor->clear();
}

QString KanaEditor::editorToPlainText(){
	return editor->toPlainText();
}

QTextDocument * KanaEditor::editorDocument (){
	return editor->document();
}

void KanaEditor::editorSetPlainText(const QString & text){
	editor->setPlainText(text);
}

void KanaEditor::prepareRootItem ( KanaListWidgetItem *item ){
	item->setFlags(Qt::ItemIsEnabled);
	QBrush brush = item->background();
	brush.setColor(Qt::lightGray);
	brush.setStyle(Qt::SolidPattern);
	item->setBackground(brush);
	QFont font = item->font();
	font.setBold(0);
	font.setPointSize(11);
	item->setFont(font);
	item->setTextAlignment(Qt::AlignHCenter);
}

void KanaEditor::on_editor_textChanged() {
// 	QMessageBox::warning(0,QObject::tr("Kana XML parser"),
// 		QObject::tr("call: on_editor_textChanged"));
	QTextCursor cursor = editor->textCursor();
	qDebug() << "got text cursor";
	cursor.movePosition( QTextCursor::PreviousCharacter, QTextCursor::KeepAnchor, 1 );
	if(cursor.hasSelection()){
		qDebug() << "Selected text is : " << cursor.selectedText();
		QString txt = cursor.selectedText() ;
		currentRomanjiWord.append(txt);
		if(txt == " " || txt == "," || txt == "." || txt == "!" || txt == "?" || txt == ";" || txt == "\n" || txt == "\r\n" ){
			cursor_start_position = cursor.position() + 1;
			currentRomanjiWord = "";
			if( currentKanaWord.length() >= 2 ){
				KanaListWidgetItem *tmp_item = new KanaListWidgetItem(0,currentRomanjiWord,currentKanaWord);
// 				QMessageBox::warning(0,QObject::tr("KanaEditor"),
// 					QObject::tr("Inserting Kana word in wordlist: %1").arg(currentKanaWord));
				kanaList->push_front(tmp_item) ;
			}
		}
		qDebug() << "currentRomanjiWord is " << currentRomanjiWord;
		cursor.movePosition( QTextCursor::NextCharacter, QTextCursor::KeepAnchor, 1 );
		cursor.movePosition( QTextCursor::PreviousCharacter, QTextCursor::KeepAnchor, cursor.position()-cursor_start_position );
		if( currentRomanjiWord == ""){
			helpListWidget->hide();
			editor->setFocus(Qt::ActiveWindowFocusReason);
		}
		else{
			bool shouldShow = createHelpList();
			QRect cursor_position = editor->cursorRect();
			QPoint br = cursor_position.bottomRight();
			helpListWidget->setGeometry( br.x() , br.y() , editor->width()/3 , editor->height()/3 );
			if( helpListWidget->isHidden() && shouldShow )
				helpListWidget->show();
			if(! helpListWidget->hasFocus() && shouldShow )
				helpListWidget->setFocus(Qt::ActiveWindowFocusReason);
			if( !shouldShow ){
				helpListWidget->hide();
				editor->setFocus(Qt::ActiveWindowFocusReason);
			}
		}
	}
	else{
		qDebug() << "no selection yet.";
		helpListWidget->hide();
		currentRomanjiWord="";
		editor->setFocus(Qt::ActiveWindowFocusReason);
	}
}

bool KanaEditor::createHelpList(){
	qDebug() << "createHelpList() the kanaList have the following size : " << kanaList->size();
	qDebug() << "helpListWidget contains " << helpListWidget->count() << " items";
	kanjiRootItem->setHidden(0);
	katakanaRootItem->setHidden(0);
	hiraganaRootItem->setHidden(0);
	sentenceRootItem->setHidden(0);
	int i=0;
	while(helpListWidget->count() != 3){
		if( helpListWidget->item(i) && helpListWidget->item(i)->text() != "Hiragana" && helpListWidget->item(i)->text() != "Katakana" && helpListWidget->item(i)->text() != "Kanji"  )
			helpListWidget->takeItem(i);
		++i;
		if(i >= helpListWidget->count())
			i=0;
	}
	qDebug() << "helpListWidget contains " << helpListWidget->count() << " items";
	bool hasHiragana = false;
	bool hasKatakana = false;
	bool hasKanji = false;
	bool hasWord = false;
	for (int i = 0; i < kanaList->size(); ++i){
		KanaListWidgetItem *item = kanaList->value(i);
		if(item->listWidget()){
			qDebug() << "item listWidget: " << item->listWidget() << " info: " << item->text() << " " << item->romanji();
			item->listWidget()->takeItem(i);
		}
		if(item->romanji().startsWith(currentRomanjiWord)){
			qDebug() << item->text() << " " << item->romanji() << " starts with " << currentRomanjiWord << " type: " << item->kanaType();
			qDebug() << "toolTip is " << item->toolTip();
			if(item->kanaType() == "kanji"){
				helpListWidget->insertItem( helpListWidget->count(), item );
				hasKanji = true;
			}
			else if(item->kanaType() == "katakana"){
				helpListWidget->insertItem( helpListWidget->row(kanjiRootItem), item );
				hasKatakana = true;
			}
			else if(item->kanaType() == "hiragana"){
				helpListWidget->insertItem( helpListWidget->row(katakanaRootItem), item );
				hasHiragana = true;
			}
			
		}
	}
	
	/* Auto-completion on full words */
	if(currentRomanjiWord.length() >= 3 ){
// 		QMessageBox::warning(0,QObject::tr("Kana XML parser"),
// 			QObject::tr("found a romanji word longer than 3"));
		for (int i = 0; i < kanaWordList->size(); ++i){
			KanaListWidgetItem *item = kanaWordList->value(i);
			if(item->listWidget()){
				qDebug() << "item listWidget: " << item->listWidget() << " info: " << item->text() << " " << item->romanji();
				item->listWidget()->takeItem(i);
			}
			if(item->romanji().startsWith(currentRomanjiWord)){
				helpListWidget->insertItem( 0, item );
				hasWord = true;
			}
		}
	}
	
	bool shouldShow = false;
	if(hasKanji){
		QListWidgetItem *it = helpListWidget->item( helpListWidget->row(kanjiRootItem)+1 );
		it->setSelected(1);
		helpListWidget->setCurrentItem(it);
		shouldShow = true;
	}
	else
		kanjiRootItem->setHidden(1);
	if(hasKatakana){
		QListWidgetItem *it = helpListWidget->item( helpListWidget->row(katakanaRootItem)+1 );
		it->setSelected(1);
		helpListWidget->setCurrentItem(it);
		shouldShow = true;
	}
	else
		katakanaRootItem->setHidden(1);
	if(hasHiragana){
		QListWidgetItem *it = helpListWidget->item( helpListWidget->row(hiraganaRootItem)+1 );
		it->setSelected(1);
		helpListWidget->setCurrentItem(it);
		shouldShow = true;
	}
	else
		hiraganaRootItem->setHidden(1);
	
	/* higly experimental code */
	if(hasWord){
		QListWidgetItem *it = helpListWidget->item( 0 );
		it->setSelected(1);
		helpListWidget->setCurrentItem(it);
		shouldShow = true;
	}
	else
		sentenceRootItem->setHidden(1);

 
// 	for(int i = 0 ; i < helpListWidget->count() ; ++i ){
// 		KanaListWidgetItem *item = static_cast<KanaListWidgetItem *>(helpListWidget->item(i));
// 		qDebug() << item->romanji() << " indexed at " << i;
// 	}
	qDebug() << "helpListWidget contains " << helpListWidget->count() << " items";
	
	return shouldShow ;
}

void KanaEditor::keyPressEvent ( QKeyEvent * event ){
	event->accept();
	bool continueEventProcessing = true;
	if( event->key() == Qt::Key_Escape){
		qDebug() << "key escape pressed.";
		helpListWidget->hide();
		editor->setFocus(Qt::ActiveWindowFocusReason);
		continueEventProcessing = false;
	}
	qDebug() << "pressed key: " << event->text();
	if(continueEventProcessing)
		QWidget::keyPressEvent(event);
}

void KanaEditor::helper_insertText(const QString& str){
	qDebug() << "receiving signal insertText(" << str << ")";
	QTextCursor cursor = editor->textCursor();
	// selecting romanji text
	cursor.movePosition( QTextCursor::PreviousCharacter, QTextCursor::KeepAnchor, cursor.position()-cursor_start_position );
	editor->setTextCursor(cursor);
	cursor.removeSelectedText();
	cursor.insertText(str);
	cursor_start_position = cursor.position();
	helpListWidget->hide();
	currentRomanjiWord="";
	editor->setFocus(Qt::ActiveWindowFocusReason);
	currentKanaWord += str; /** suuuuucks !! **/
}

void KanaEditor::helpListWidget_itemActivated(QListWidgetItem *item){
	/** IMPORTANT **/
	/** 
	the full words completion do not works because currentRomanjiWord contains only 
	the romanji for the latest kana being processed (those printed in the editor)
	NOT THE FULL WORD !!
	I need to concatenate the romanji of the item here (so cast it into a KanaListWidgetItem)
	**/
	/** IMPORTANT **/
	helper_insertText( item->text() );
}

void KanaEditor::on_editor_editorClicked(){
	helpListWidget->hide();
	editor->setFocus(Qt::ActiveWindowFocusReason);
}

void KanaEditor::on_editor_backspacePressed(){
	/* Here we prevent the on_editor_textChanged() slot to double the last letter */
	currentRomanjiWord = 
currentRomanjiWord.remove( currentRomanjiWord.length()-2 , 2 );
	createHelpList();
}

// void KanaEditor::on_helpListWidget_currentRowChanged ( int currentRow ) {
// 	QListWidgetItem *item = helpListWidget->item(currentRow);
// 	qDebug() << QString("currentRowChanged, current item is: %1").arg(item->text());
// 	if( item->text() == "Hiragana" || item->text() == "Katakana" || item->text() == "Kanji" || item->text() == tr("Full words") ){
// 		QListWidgetItem *it = helpListWidget->item(currentRow+1);
// 		it->setSelected(1);
// 		helpListWidget->setCurrentItem(it);
// 	}
// }
// 
// 
//  void KanaEditor::on_helpListWidget_currentItemChanged ( QListWidgetItem * current, QListWidgetItem * previous ) {
//  	QMessageBox::warning(0,QObject::tr("Kana XML parser"),
// 	QObject::tr("currentItemChanged, current item is: %1").arg(current->text()));
//  }



