#include "kanaconfigdialog.h"
#include <QSettings>
#include <QFileDialog>
#include <QProgressDialog>
#include <QDebug>
#include <QMessageBox>
#include <QTextCodec>
#include "KanaListWidgetItem.h"

KanaConfigDialog::KanaConfigDialog(QWidget *parent) : QDialog(parent) {
	setupUi(this);
	kanaList = new QList<KanaListWidgetItem *>;
	
	readSettings();
}

void KanaConfigDialog::on_resComboBox_activated(const QString& text){
	if( text == tr("Custom...") ){
		label_2->setEnabled(true);
		lineEdit->setEnabled(true);
		toolButton->setEnabled(true);
		addKanaRowButton->setEnabled(true);
		removeKanaRowButton->setEnabled(true);
	}
	else{
		label_2->setEnabled(false);
		lineEdit->setEnabled(false);
		toolButton->setEnabled(false);
		lineEdit->setText(":/data.xml");
		addKanaRowButton->setEnabled(false);
		removeKanaRowButton->setEnabled(false);
	}
}

void KanaConfigDialog::on_addKanaRowButton_clicked(){
	tableWidget->insertRow( tableWidget->currentRow()+1 );
}

void KanaConfigDialog::on_removeKanaRowButton_clicked(){
	tableWidget->removeRow( tableWidget->currentRow() );
}

void KanaConfigDialog::loadDataFile(const QString& fileName) {
	QFile file(fileName);
	QXmlInputSource inputSource(&file);
	QXmlSimpleReader reader;
	kanaList->clear();
	KanaXmlHandler handler(kanaList);
	reader.setContentHandler(&handler);
	reader.setErrorHandler(&handler);
	reader.parse(inputSource);
	QProgressDialog progress( tr("Please wait while loading data file."), tr("Cancel"), 0, kanaList->count()-1, this );
	for( int i = 0 ; i < kanaList->count(); ++i ){
		progress.setValue(i);
		KanaListWidgetItem *it = kanaList->value(i);
		QList<QTableWidgetItem *> list = tableWidget->findItems( it->text() , Qt::MatchExactly );
		if(list.count() == 1 ){
			int item_row = tableWidget->row(list.value(0));
			QTableWidgetItem * tmp_item = tableWidget->item(item_row, 0);
			tmp_item->setText( tmp_item->text() + "|" + it->romanji() );
			tableWidget->setItem( i, 0 , tmp_item );
		}
		else{
			int row_count = tableWidget->rowCount();
			tableWidget->insertRow(row_count);
			tableWidget->setItem( row_count, 0 , new QTableWidgetItem( it->romanji() ) );
			tableWidget->setItem( row_count, 1 , new QTableWidgetItem( it->text() ) );
			tableWidget->setItem( row_count, 2 , new QTableWidgetItem( it->kanaType() ) );
			tableWidget->setItem( row_count, 3 , new QTableWidgetItem( it->rawTip() ) );
		}
		if (progress.wasCanceled())
			break;
	}
}

void KanaConfigDialog::accept(){
	writeSettings();
	if( !lineEdit->text().startsWith(":/") )
		writeData( lineEdit->text() );
	emit configurationUpdated();
	QDialog::accept();
}

void KanaConfigDialog::writeData(const QString & dataFileName){
	QFile file(dataFileName);
	if (!file.open(QFile::WriteOnly | QFile::Text)) {
		QMessageBox::warning(this, tr("The \"P\" Kana Editor"),
				tr("Cannot write file %1:\n%2.")
				.arg(dataFileName)
				.arg(file.errorString()));
	}
	
	QTextStream out(&file);
	out.setCodec(QTextCodec::codecForName("UTF-8"));
	
	QApplication::setOverrideCursor(Qt::WaitCursor);
	out << "<?xml version=\"1.0\" encoding=\"utf-8\" standalone=\"yes\"?>\n<kanalist>\n";
	QApplication::restoreOverrideCursor();
	
	for (int i = 0 ; i < tableWidget->rowCount(); ++i ){
		QTableWidgetItem *it_romanji = tableWidget->item(i,0);
		QTableWidgetItem *it_kana = tableWidget->item(i,1);
		QTableWidgetItem *it_type = tableWidget->item(i,2);
		QTableWidgetItem *it_tooltip = tableWidget->item(i,3);
		QString html_tip = it_tooltip->text();
		html_tip = html_tip.replace("&","&amp;");
		html_tip = html_tip.replace("<","&lt;");
		html_tip = html_tip.replace(">","&gt;");
		
		out << "     <item romanji=\"" << it_romanji->text() << "\" kana=\"" << it_kana->text() << "\" type=\"" << it_type->text() << "\" tips=\"" << html_tip << "\" />\n";
	}
	out << "</kanalist>\n";
}

void KanaConfigDialog::on_toolButton_clicked(){
	QString file = QFileDialog::getOpenFileName(this,
		tr( "Open data file" ),
		QDir::currentPath()
	);
	if(!file.isEmpty())
		lineEdit->setText(file);
}

void KanaConfigDialog::readSettings()
{
	QSettings settings("InfinityPerl", "PKanaEditor");
	QString curDataFile = settings.value("dataFile",":/data.xml").toString();
	if( curDataFile == ":/data.xml" ){
		resComboBox->setCurrentIndex( resComboBox->findText( tr("Embed") ) );
		on_resComboBox_activated(tr("Embed"));
	}
	else{
		resComboBox->setCurrentIndex( resComboBox->findText( tr("Custom...") ) );
		on_resComboBox_activated(tr("Custom..."));
	}
	lineEdit->setText( curDataFile );
	loadDataFile( curDataFile );
}

void KanaConfigDialog::writeSettings()
{
	QSettings settings("InfinityPerl", "PKanaEditor");
	settings.setValue("dataFile",lineEdit->text());
}
